package lib

// Various constants used throughout the tools.

const (
	// ExitSuccess is the successful exit status.
	//
	// It should be called on successful exit.
	ExitSuccess = 0

	// ExitFailure is the failing exit status.
	ExitFailure = 1
)
